r"""py-deps provides parsing the Python deps and generating graph data.

Search packages
---------------

Search packages from PyPI.::

    $ python
    >>> from py_deps import Package
    >>> Package.search('deps')
    [{'_pypi_ordering': False,
      'name': 'anybox.recipe.sysdeps',
      'summary': 'A buildout recipe to check system dependencies',
      'version': '0.5'},
     {'_pypi_ordering': False,
      'name': 'appdynamics-bindeps-linux-x64',
      'summary': 'Dependencies for AppDynamics Python agent',
      'version': '4.0.5.0'},
     {'_pypi_ordering': False,
      'name': 'appdynamics-bindeps-linux-x86',
      'summary': 'Dependencies for AppDynamics Python agent',
      'version': '4.0.5.0'},
     {'_pypi_ordering': False,
      'name': 'appdynamics-bindeps-osx-x64',
      'summary': 'Dependencies for AppDynamics Python agent',
      'version': '4.0.5.0'},
     {'_pypi_ordering': False,
      'name': 'deps',
      'summary': 'deps discovers your Python dependencies',
      'version': '0.1.0'},
     {'_pypi_ordering': False,
      'name': 'gtkeggdeps',
      'summary': 'Interactive egg dependency browser',
      'version': '0.0.7'},
     {'_pypi_ordering': False,
      'name': 'htmldeps',
      'summary': 'Expand CSS and javascript dependency links in HTML',
      'version': '1.2.1'},
     {'_pypi_ordering': False,
      'name': 'py-deps',
      'summary': 'parsing the Python deps and generating graph data',
      'version': '0.3.0'},
     {'_pypi_ordering': False,
      'name': 'pydeps',
      'summary': 'Display module dependencies',
      'version': '1.2.1'},
     {'_pypi_ordering': False,
      'name': 'runestone-test-deps',
      'summary': 'This is dependencies for RSI',
      'version': '0.1'},
     {'_pypi_ordering': False,
      'name': 'tl.eggdeps',
      'summary': 'Compute a dependency graph between active Python eggs.',
      'version': '0.4'},
     {'_pypi_ordering': False,
      'name': 'tt.eggdeps',
      'summary': 'Compute a dependency graph between active Python eggs.',
      'version': '0.5'}]


Initialize
----------

Cache the parsed dependencies into the ``py-deps.pickle``
on current working directory.This file format is
`pickle <https://docs.python.org/2.7/library/pickle.html>`_.::

    $ python
    >>> from py_deps import Package
    >>> pkg = Package('py-deps')


Change cache file
~~~~~~~~~~~~~~~~~

Use cache_name argument.::

    >>> pkg = Package('py-deps', cache_name='some-cache.name')


Override cache forcely
~~~~~~~~~~~~~~~~~~~~~~

Use ``update_force`` argument. (default: ``False``)::

    >>> pkg = Package('py-deps', update_force=True)


Generate rendering data
-----------------------

Supports follows currently.

* pretty print
* Linkdraw

Pretty print
~~~~~~~~~~~~

::

    >>> print(pkg.draw())
    py-deps -> [Sphinx, setuptools, pip, wheel, tox]
    setuptools -> [certifi, wincertstore, setuptools[ssl], pytest]
    pip -> [pytest, virtualenv, scripttest, mock, pytest, virtualenv,
     scripttest, mock]
    wheel -> [ed25519ll, keyring, argparse, pyxdg, jsonschema, pytest,
     coverage, pytest-cov]
    >>>

Linkdraw
~~~~~~~~

::

    >>> import json
    >>> json.loads(pkg.draw('linkdraw'))
    {u'descr': u'py-deps dependencies',
     u'lines': [{u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'py-deps', u'target': u'Sphinx', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'py-deps', u'target': u'setuptools', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'py-deps', u'target': u'pip', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'py-deps', u'target': u'wheel', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'py-deps', u'target': u'tox', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'setuptools', u'target': u'certifi', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'setuptools', u'target': u'wincertstore', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'setuptools', u'target': u'setuptools____ssl',
       u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'setuptools', u'target': u'pytest', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'pytest', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'virtualenv', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'scripttest', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'mock', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'pytest', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'virtualenv', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'scripttest', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'pip', u'target': u'mock', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'ed25519ll', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'keyring', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'argparse', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'pyxdg', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'jsonschema', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'pytest', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'coverage', u'width': u'1'},
      {u'color': u'#5F9EA0', u'descr': u'->', u'link': u'',
       u'source': u'wheel', u'target': u'pytest-cov', u'width': u'1'}],
     u'nodes': [{u'color': u'', u'link': u'https://github.com/mkouhei/py-deps',
       u'name': u'py-deps', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'Sphinx', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'https://bitbucket.org/pypa/setuptools',
       u'name': u'setuptools', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'https://pip.pypa.io/',
       u'name': u'pip', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'http://bitbucket.org/pypa/wheel/',
       u'name': u'wheel', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'tox', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'certifi', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'wincertstore',
       u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'setuptools____ssl',
       u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'pytest', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'virtualenv', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'scripttest', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'mock', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'ed25519ll', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'keyring', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'argparse', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'pyxdg', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'jsonschema', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'coverage', u'r': u'6'},
      {u'color': u'#5F9EA0', u'link': u'', u'name': u'pytest-cov',
       u'r': u'6'}],
     u'time': u'2015-05-08T03:52:59.542732'}


See also `How to use linkdraw
<https://github.com/mtoshi/linkdraw/wiki#how-to-use-linkdraw>`_.


NetworkX
~~~~~~~~

::

    >>> pkg.draw('networkx')
    >>> <networkx.classes.digraph.DiGraph at 0x7fbe2311dbd0>


Check cache
-----------

Stores parsed dependency metadata to pickles data file.
The file name is ``py-deps.pickle`` in default.

Listing cached data with the ``list_data`` method of :class:`Container`.::

    >>> from py_deps import Container
    >>> Container().list_data()
    {'py-deps': [py-deps, setuptools, pip, wheel, networkx, decorator],
     (snip)}

Read the cached package with ``read_data`` method of :class:`Container`.
This method returns :class:`Package.traced_chain`.

    >>> Container().read_data('py-deps')
    [py-deps, setuptools, pip, wheel, networkx, decorator]


"""
from py_deps.deps import Package, Container  # silence pyflakes



# -*- coding: utf-8 -*-
"""py_deps.graph module."""
import json
import networkx
from datetime import datetime


def router(chain_data, draw_type=None):
    """Routing drawing tool."""
    if draw_type == 'networkx':
        nwx = Networkx(chain_data)
        return nwx.generate_data()
    elif draw_type == 'blockdiag':
        pass
    elif draw_type == 'linkdraw':
        linkdraw = Linkdraw(chain_data)
        return linkdraw.generate_data()
    else:
        return pretty_print(chain_data)


def pretty_print(chain_data):
    """Pretty print on terminal.

    :param list chain_data: List of `deps.Node`
    """
    lines = ''
    for node in chain_data:
        lines += '%s -> %s\n' % (node, node.targets)
    return lines.rstrip()


class Graph(object):

    """Graph data generate abstract class."""

    default_radius = "6"
    default_color = ""
    requires_color = "#5F9EA0"
    default_width = "1"

    def __init__(self, chain_data):
        """Initialize."""
        self.chain_data = chain_data

    def _generate_node(self, node, nodes):
        """Generate node data."""
        if self._check_node(node, nodes):
            metadata = self._get_metadata(node.name)
            nodes.append(dict(name=self._normalize_name(node.name),
                              r=self.default_radius,
                              color=self.default_color,
                              version=metadata.version,
                              link=self._normalize_url(metadata.url)))
        if len(node.targets) > 0:
            for target in node.targets:
                if self._check_node(target, nodes):
                    metadata = self._get_metadata(target.name)
                    nodes.append(dict(name=self._normalize_name(target.name),
                                      r=self.default_radius,
                                      color=self.requires_color,
                                      version=metadata.version,
                                      link=self._normalize_url(metadata.url)))

    def _check_node(self, node, nodes):
        """Check appended node."""
        if len([_node for _node in nodes
                if _node['name'] == self._normalize_name(node.name)]) == 0:
            return True
        else:
            return False

    def generate_nodes(self):
        """Generate nodes data."""
        nodes = []
        for node in self.chain_data:
            self._generate_node(node, nodes)
        return nodes

    @staticmethod
    def _normalize_url(url):
        """return package url."""
        if url is None:
            return ''
        else:
            return url

    @staticmethod
    def _normalize_name(name):
        """Normalize name."""
        return name

    def _get_metadata(self, name):
        """get the metadata of package."""
        data = [req for req in self.chain_data if req.name == name]
        if len(data) == 1:
            return data[0]
        else:
            return Metadata()


class Metadata(object):

    """Metadata object class."""

    version = None
    url = None


class Linkdraw(Graph):

    """Linkdraw object class."""

    def __init__(self, chain_data):
        """Initialize."""
        super(Linkdraw, self).__init__(chain_data)
        self.descr = "%s dependencies" % chain_data[0].name

    def generate_edges(self):
        """Generate edges data."""
        return [dict(source=self._normalize_name(node.name),
                     target=self._normalize_name(target.name),
                     color=self.requires_color,
                     width=self.default_width,
                     descr="->",
                     link="")
                for node in self.chain_data
                for target in node.targets]

    def generate_data(self):
        """Generate Linkdraw data."""
        return json.dumps(dict(time=datetime.utcnow().isoformat(),
                               descr=self.descr,
                               nodes=self.generate_nodes(),
                               lines=self.generate_edges()))

    @staticmethod
    def _normalize_name(name):
        """Normalize name."""
        return name.replace('[', '____').replace(']', '')


class Networkx(Graph):

    """Networkx object class."""

    def __init__(self, chain_data):
        """Initialize."""
        super(Networkx, self).__init__(chain_data)
        self.graph = networkx.DiGraph()

    def generate_edges(self):
        """Generate edges data."""
        self.graph.add_edges_from([(self._normalize_name(node.name),
                                    self._normalize_name(target.name))
                                   for node in self.chain_data
                                   for target in node.targets],
                                  color=self.requires_color)

    def generate_data(self):
        """Generate networkx graph data."""
        for node in self.generate_nodes():
            self.graph.add_node(node['name'],
                                version=node['version'],
                                link=node['link'])
        self.generate_edges()
        return self.graph



# -*- coding: utf-8 -*-
"""py_deps.tests.test_deps module."""
import sys
import unittest
import os
import tempfile
import shutil
import json
from glob import glob
from mock import patch
from py_deps import deps


@patch('pip.req.RequirementSet.prepare_files')
def prepare(pkg_name, meta_type, cache, _mock):
    """Prepare package object."""
    pkg = deps.Package(pkg_name)
    for meta in glob('py_deps/tests/data/meta/%s/*' % meta_type):
        shutil.copytree(meta,
                        os.path.join(pkg.tempdir,
                                     os.path.basename(meta)))
    pkg.trace_chain()
    cache.store_data(pkg_name, pkg.traced_chain)
    return pkg


def search_result():
    """generate dummy result."""
    with open('py_deps/tests/data/search_result') as fobj:
        return json.loads(fobj.read())


class SearchTests(unittest.TestCase):

    """Test of search via XMLRPC."""

    if sys.version_info < (3, 0):
        @patch('xmlrpclib.ServerProxy')
        def test_search(self, _mock):
            """search package."""
            client_mock = _mock.return_value
            client_mock.search.return_value = search_result()
            self.assertListEqual(deps.Package.search('deps'), search_result())

        @patch('xmlrpclib.ServerProxy')
        def test_search_exactly(self, _mock):
            """search package exactly."""
            client_mock = _mock.return_value
            client_mock.search.return_value = search_result()
            self.assertListEqual(deps.Package.search('py-deps', exactly=True),
                                 [search_result()[8]])

    if sys.version_info > (3, 0):
        @patch('xmlrpc.client.ServerProxy')
        def test_search_py3(self, _mock):
            """search package."""
            client_mock = _mock.return_value
            client_mock.search.return_value = search_result()
            self.assertListEqual(deps.Package.search('deps'), search_result())

        @patch('xmlrpc.client.ServerProxy')
        def test_search_exactly_py3(self, _mock):
            """search package exactly."""
            client_mock = _mock.return_value
            client_mock.search.return_value = search_result()
            self.assertListEqual(deps.Package.search('py-deps', exactly=True),
                                 [search_result()[8]])


class WheelTests(unittest.TestCase):

    """Test of Package class."""

    cache = 'py-deps.tests'

    def setUp(self):
        """Initialize."""
        # self.maxDiff = None
        with open('py_deps/tests/data/wheel.pretty_print') as fobj:
            self.pretty_print = fobj.read()

        with open('py_deps/tests/data/wheel.linkdraw') as fobj:
            self.linkdraw = json.loads(fobj.read())

        deps.DEFAULT_CACHE_NAME = self.cache
        self.container = deps.Container(self.cache)
        self.pkg = prepare('py-deps', 'wheel', self.container)
        self.tempdir = tempfile.mkdtemp(suffix=deps.SUFFIX)

    def tearDown(self):
        """Clean up test."""
        self.pkg.cleanup()
        os.remove(deps.DEFAULT_CACHE_NAME)

    def test_pretty_print(self):
        """Pretty print test."""
        self.assertEqual(self.pkg.draw(),
                         self.pretty_print)

        # cache test
        pkg_cache = deps.Package('py-deps',
                                 cache_name=self.cache)
        self.assertEqual(pkg_cache.draw(),
                         self.pretty_print)
        self.assertEqual(len(self.container.list_data().get('py-deps')), 4)

    def test_linkdraw(self):
        """Linkdraw test."""
        data = json.loads(self.pkg.draw('linkdraw'))
        data['time'] = None
        self.linkdraw['time'] = None
        self.assertEqual(data, self.linkdraw)

    def test_networkx(self):
        """Networkx test."""
        self.assertEqual(len(self.pkg.draw('networkx').nodes()), 20)
        self.assertEqual(len(self.pkg.draw('networkx').edges()), 21)

    def test_cleanup_all(self):
        """Cleanup tests."""
        self.pkg.cleanup(alldir=True)
        self.assertListEqual(
            glob('%s/tmp*%s' % (os.path.dirname(self.tempdir), deps.SUFFIX)),
            [])


class WheelDeprecatedTests(unittest.TestCase):

    """Test of Package class for old style wheel."""

    cache = 'py-deps.tests'

    def setUp(self):
        """Initialize."""
        # self.maxDiff = None
        with open('py_deps/tests/data/wheel-deprecated.pretty_print') as fobj:
            self.pretty_print = fobj.read()

        with open('py_deps/tests/data/wheel-deprecated.linkdraw') as fobj:
            self.linkdraw = json.loads(fobj.read())

        deps.DEFAULT_CACHE_NAME = self.cache
        self.container = deps.Container(self.cache)
        self.pkg = prepare('iso8601', 'wheel-deprecated', self.container)
        self.tempdir = tempfile.mkdtemp(suffix=deps.SUFFIX)

    def tearDown(self):
        """Clean up test."""
        self.pkg.cleanup()
        os.remove(deps.DEFAULT_CACHE_NAME)

    def test_pretty_print(self):
        """Pretty print test."""
        self.assertEqual(self.pkg.draw(),
                         self.pretty_print)

        # cache test
        pkg_cache = deps.Package('iso8601',
                                 cache_name=self.cache)
        self.assertEqual(pkg_cache.draw(),
                         self.pretty_print)
        self.assertEqual(len(self.container.list_data().get('iso8601')), 1)

    def test_linkdraw(self):
        """Linkdraw test."""
        data = json.loads(self.pkg.draw('linkdraw'))
        data['time'] = None
        self.linkdraw['time'] = None
        self.assertEqual(data, self.linkdraw)

    def test_networkx(self):
        """Networkx test."""
        self.assertEqual(len(self.pkg.draw('networkx').nodes()), 1)
        self.assertEqual(len(self.pkg.draw('networkx').edges()), 0)

    def test_cleanup_all(self):
        """Cleanup tests."""
        self.pkg.cleanup(alldir=True)
        self.assertListEqual(
            glob('%s/tmp*%s' % (os.path.dirname(self.tempdir), deps.SUFFIX)),
            [])


class EggTests(unittest.TestCase):

    """Test of Package class."""

    cache = 'py-deps.tests'

    def setUp(self):
        """Initialize."""
        # self.maxDiff = None
        with open('py_deps/tests/data/egg.pretty_print') as fobj:
            self.pretty_print = fobj.read()

        with open('py_deps/tests/data/egg.linkdraw') as fobj:
            self.linkdraw = json.loads(fobj.read())

        deps.DEFAULT_CACHE_NAME = self.cache
        self.container = deps.Container(self.cache)
        self.pkg = prepare('swiftsc', 'egg', self.container)
        self.tempdir = tempfile.mkdtemp(suffix=deps.SUFFIX)

    def tearDown(self):
        """Clean up test."""
        self.pkg.cleanup()
        os.remove(deps.DEFAULT_CACHE_NAME)

    def test_pretty_print(self):
        """Pretty print test."""
        self.assertEqual(self.pkg.draw(),
                         self.pretty_print)

        # cache test
        pkg_cache = deps.Package('swiftsc',
                                 cache_name=self.cache)
        self.assertEqual(pkg_cache.draw(),
                         self.pretty_print)
        self.assertEqual(len(self.container.list_data().get('swiftsc')), 4)

    def test_linkdraw(self):
        """Linkdraw test."""
        data = json.loads(self.pkg.draw('linkdraw'))
        data['time'] = None
        self.linkdraw['time'] = None
        self.assertEqual(data, self.linkdraw)

    def test_networkx(self):
        """Networkx test."""
        self.assertEqual(len(self.pkg.draw('networkx').nodes()), 11)
        self.assertEqual(len(self.pkg.draw('networkx').edges()), 10)

    def test_cleanup_all(self):
        """Cleanup tests."""
        self.pkg.cleanup(alldir=True)
        self.assertListEqual(
            glob('%s/tmp*%s' % (os.path.dirname(self.tempdir), deps.SUFFIX)),
            [])



# -*- coding: utf-8 -*-
"""py_deps.exceptions module."""


class Error(Exception):

    """Base error class."""

    def __init__(self, message=None):
        """Initialize."""
        super(Error, self).__init__(message)


class NotFound(Error):

    """Not Found."""

    pass


class BrokenPackage(Error):

    """BrokenPackage."""

    pass



# -*- coding: utf-8 -*-
"""py_deps.logger module."""
import sys
import logging


def trace_log(level='info'):
    """Traceback loggging."""
    _type, _val, _tb = sys.exc_info()
    msg = ("%s\n  CODE: %s\n  LINE: %s\n  NAME: %s\n  MSG : %s" %
           (_type,
            _tb.tb_frame.f_code.co_filename,
            _tb.tb_lineno,
            _tb.tb_frame.f_code.co_name,
            _val))
    if level == 'info':
        logging.info(msg)
    elif level == 'warning':
        logging.warning(msg)
    elif level == 'error':
        logging.error(msg)
    elif level == 'critical':
        logging.critical(msg)
    elif level == 'debug':
        logging.debug(msg)



# -*- coding: utf-8 -*-
"""py_deps.deps module."""
import sys
import os
import tempfile
import re
import pip
import wheel.util
from glob import glob
from pip.req import RequirementSet, InstallRequirement
from pip.locations import src_prefix
from pip.download import PipSession
from pip.index import PackageFinder
from pkg_resources import PathMetadata, Distribution
import pickle
from py_deps import graph
from py_deps.exceptions import NotFound, BrokenPackage
from py_deps.logger import trace_log
if pip.__version__ >= '6.0.0':
    from pip.utils import rmtree
else:
    from pip.util import rmtree
if sys.version_info > (3, 0):
    import xmlrpc.client as xmlrpclib
else:
    import xmlrpclib


#: suffix of temporary directory name
SUFFIX = '-py_deps'
#: default cache file name
DEFAULT_CACHE_NAME = 'py-deps.pickle'
PYPI_URL = 'https://pypi.python.org/pypi'


class Container(object):

    """Package container class."""

    def __init__(self, cache_name=None):
        """Initialize."""
        if cache_name:
            self.cache_name = cache_name
        else:
            self.cache_name = DEFAULT_CACHE_NAME
        self.container = {}
        self.load_cache()

    def load_cache(self):
        """Load cache file."""
        if os.path.isfile(self.cache_name):
            with open(self.cache_name, 'rb') as fobj:
                self.container = pickle.load(fobj)

    def save_cache(self):
        """Save cache file."""
        with open(self.cache_name, 'wb') as fobj:
            pickle.dump(self.container, fobj)

    def read_data(self, key):
        """Read traced_chain data.

        :rtype: list
        :return: dependency chain list

        :param str key: package name
        """
        return self.container.get(key)

    def store_data(self, key, data):
        """Store traced_chain data.

        :param str key: package name
        :param list data: traced dependency chain data
        """
        self.container[key] = data
        self.save_cache()

    def list_data(self):
        """Return dictionary stored package metadata.

        :rtype: dict
        :return: packages metadata
        """
        return self.container


class Package(object):

    """Package class."""

    #: index_url
    index_url = 'https://pypi.python.org/simple'

    def __init__(self, name, cache_name=None, update_force=False):
        """Initialize to parsing dependencies of package."""
        #: package name
        self.name = name
        cache = Container(cache_name)
        self.container = cache.container
        self.tempdir = tempfile.mkdtemp(suffix=SUFFIX)

        if cache.read_data(name) is None or update_force:

            self.finder = PackageFinder(find_links=[],
                                        index_urls=[self.index_url],
                                        session=PipSession())
            self.reqset = RequirementSet(build_dir=self.tempdir,
                                         src_dir=src_prefix,
                                         download_dir=None,
                                         upgrade=True,
                                         ignore_installed=True,
                                         session=PipSession())

            req = InstallRequirement.from_line(name,
                                               comes_from=None)
            self.reqset.add_requirement(req)
            self.requires = []
            self.traced_chain = []
            self.trace_chain()
            cache.store_data(name, self.traced_chain)
        else:
            self.traced_chain = cache.read_data(name)
            self.cleanup()

    @classmethod
    def search(cls, pkg_name, exactly=False):
        """Search package.

        :rtype: list
        :return: search packages

        :param str pkg_name: package name.
        :param bool exactly: exactly match only.
        """
        client = xmlrpclib.ServerProxy(PYPI_URL)
        result = client.search({'name': pkg_name})
        if exactly:
            return [pkg for pkg in result
                    if u2h(pkg.get('name')) == u2h(pkg_name)]
        else:
            return result

    def cleanup(self, alldir=False):
        """Cleanup temporary build directory.

        :param bool alldir: Remove all temporary directories. (default: False)

        :rtype: None
        """
        if alldir:
            for tempdir in glob("%s/tmp*%s" % (os.path.dirname(self.tempdir),
                                               SUFFIX)):
                rmtree(tempdir, ignore_errors=True)
        else:
            rmtree(self.tempdir, ignore_errors=True)

    def _download(self):
        """Download packages to build_dir.

        This method does not download requires recursively.
        """
        try:
            self.reqset.prepare_files(self.finder)
        except pip.exceptions.DistributionNotFound as exp:
            trace_log(level='warning')
            raise NotFound(exp)
        except pip.exceptions.InstallationError as exp:
            trace_log(level='error')
            raise BrokenPackage(exp)

    def _list_requires(self):
        """Listing requires object or dict object.

        :rtype: list
        :return: requires object or dict object
        """
        if not self.requires:
            self._download()
            self._collect_requires()
            self.cleanup()
        return self.requires

    def trace_chain(self, pkg_name=None):
        """Trace dependency chain.

        :param str pkg_name: package name
        """
        if pkg_name is None:
            pkg_name = self.name
        pkg = [req for req in self._list_requires()
               if u2h(req.name) == u2h(pkg_name)]
        if len(pkg) == 0:
            return
        if pkg[0] not in self.traced_chain:
            self.traced_chain.append(pkg[0])
        if len(pkg[0].targets) > 0:
            for target in pkg[0].targets:
                self.trace_chain(target.name)

    def _collect_requires(self):
        """Collect requires object or dict object."""
        self.requires = [self._parse(os.path.join(self.tempdir, d))
                         for d in os.listdir(self.tempdir)
                         if os.path.isdir(os.path.join(self.tempdir, d))]

    def _parse(self, pkg_dir):
        """Parsing package metadata.

        :rtype: `pkg_resouces.Distributions` or list
        :return: package metadata

        :param str pkg_dir: package directory in build_dir.
        """
        pat = re.compile(r'(\A(.*\.(egg|dist)-info|pip-egg-info)\Z)')
        egg_dirs = [f for f in os.listdir(os.path.join(self.tempdir, pkg_dir))
                    if pat.search(f)]

        if 'pip-egg-info' in egg_dirs:
            eggs = glob('%s/*' % os.path.join(pkg_dir, 'pip-egg-info'))
            metadata = self._parse_egg_info(eggs[0])
        else:
            dist_info = os.path.join(pkg_dir, egg_dirs[0])
            metadata = self._parse_dist_info(dist_info)
        return metadata

    @staticmethod
    def _parse_egg_info(egg_info):
        """Parse ``.egg-info`` directory.

        :rtype: dict
        :return: dictionary of requires

        :param str egg_info: ``.egg-info`` directory path
        """
        base_dir = os.path.dirname(egg_info)
        metadata = PathMetadata(base_dir, egg_info)
        dist_name = os.path.splitext(os.path.basename(egg_info))[0]
        dist_obj = Distribution(base_dir,
                                project_name=dist_name,
                                metadata=metadata)
        return _dist_to_node(dist_obj)

    @staticmethod
    def _parse_dist_info(dist_info):
        """Parse ``.dist-info`` directory.

        :rtype: dict
        :return: dictionary of requires and test_requires

        :param str dist_info: ``.dist-info`` directory path.
        """
        if os.path.isfile(os.path.join(dist_info, 'metadata.json')):
            with open(os.path.join(dist_info, 'metadata.json'), 'r') as fobj:
                data = fobj.read()
        elif os.path.isfile(os.path.join(dist_info, 'pydist.json')):
            with open(os.path.join(dist_info, 'pydist.json'), 'r') as fobj:
                data = fobj.read()
        metadata = wheel.util.from_json(data)
        if metadata.get('run_requires') is None:
            # To Do: parsing requirements.txt later.
            pass
        return _wheel_to_node(metadata)

    def draw(self, draw_type=None):
        """Generate drawing data.

        :param str draw_type: [dot|blockdiag|linkdraw]
        """
        return graph.router(self.traced_chain, draw_type=draw_type)


def _dist_to_node(dist_obj):
    """Convert distribution metadata to Node objects."""
    node = Node(dist_obj.project_name, version=dist_obj.version)
    node.add_targets([Target(req.project_name, req.specs)
                      for req in dist_obj.requires()])
    return node


def _wheel_to_node(metadata):
    """Convert wheel metadata to Node objects."""
    if metadata.get('extensions'):
        url = (metadata.get('extensions')
               .get('python.details')
               .get('project_urls')
               .get('Home'))
    elif metadata.get('project_urls'):
        url = metadata.get('project_urls').get('Home')
    node = Node(metadata.get('name'),
                version=metadata.get('version'),
                url=url)
    if metadata.get('run_requires'):
        for requires in metadata.get('run_requires'):
            node.add_targets(_parse_require(requires.get('requires')))
    if metadata.get('test_requires'):
        for requires in metadata.get('test_requires'):
            node.add_targets(
                _parse_require(requires.get('requires'), extras=True))
    return node


def _parse_require(requires, extras=False):
    """parse require metadata."""
    targets = []
    for req in requires:
        if len(req.split()) == 1:
            targets.append(Target(req, None, extras))
        else:
            targets.append(Target(req.split()[0], req.split()[1], extras))
    return targets


def u2h(name):
    """Change underscore to hyphen of package name.

    :rtype: str
    :return: string replaced underscore with hyphen

    :param str name: package name
    """
    return name.replace('_', '-')


class Node(object):

    """Node object class."""

    def __init__(self, name, version=None, url=None):
        """Initialize."""
        self.name = name
        self.version = version
        self.url = url
        self.targets = []
        self.test_targets = []

    def __repr__(self):
        """Return Node object name."""
        return str(self.name)

    def add_targets(self, nodes):
        """Add targets."""
        self.targets += nodes

    def remove_targets(self, *nodes):
        """Remove targets."""
        for node in nodes:
            del self.targets[self.targets.index(node)]

    def add_test_targets(self, nodes):
        """Add test targets."""
        self.test_targets += nodes


class Target(Node):

    """Target objects."""

    def __init__(self, nodename, specs, extras=False):
        """Initialize."""
        super(Target, self).__init__(nodename)
        #: specs
        self.specs = specs
        #: extras: True is extras when test_requries (False in default)
        self.extras = extras



# -*- coding: utf-8 -*-
"""
Copyright (C) 2015 Kouhei Maeda <mkouhei@palmtb.net>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import sys
from setuptools import setup
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    user_options = [('tox-args=', 'a', 'Arguments to pass to tox')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.tox_args = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox
        import shlex
        if self.tox_args:
            errno = tox.cmdline(args=shlex.split(self.tox_args))
        else:
            errno = tox.cmdline(self.test_args)
        sys.exit(errno)


def read_content(filepath):
    with open(filepath) as fobj:
        return fobj.read()


classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: "
    "GNU General Public License v3 or later (GPLv3+)",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
]


long_description = (
    read_content("README.rst") +
    read_content(os.path.join("docs/source", "HISTORY.rst")))

requires = ['setuptools',
            'pip>=6.0.1',
            'wheel',
            'networkx']
extras_require = {
    'reST': ['Sphinx'],
    }
if os.environ.get('READTHEDOCS', None):
    extras_require['reST'].append('recommonmark')

with open('requirements.txt', 'w') as _file:
    _file.write('\n'.join(requires))

with open('extras_requirement.txt', 'w') as _file:
    _file.write('\n'.join(extras_require.get('reST')))

setup(name='py-deps',
      version='0.4.2',
      description='parsing the Python deps and generating graph data',
      long_description=long_description,
      author='Kouhei Maeda',
      author_email='mkouhei@palmtb.net',
      url='https://github.com/mkouhei/py-deps',
      license='GNU General Public License version 3',
      classifiers=classifiers,
      packages=['py_deps'],
      data_files=[],
      install_requires=requires,
      include_package_data=True,
      extras_require=extras_require,
      tests_require=['tox'],
      cmdclass={'test': Tox},)
